"use strict";
/**
 * @file This module makes available a debugging utility for `elm-ts` applications.
 *
 * `elm-ts` ships with a [Redux DevTool Extension](https://github.com/zalmoxisus/redux-devtools-extension) integration, falling back to a simple debugger via standard browser's [`console`](https://developer.mozilla.org/en-US/docs/Web/API/Console) in case the extension is not available.
 *
 * **Note:** debugging is to be considered unsafe by design so it should be used only in **development**.
 *
 * This is an example of usage:
 * ```ts
 * import {react, cmd} from 'elm-ts'
 * import {programWithDebugger} from 'elm-ts/lib/Debug'
 * import { render } from 'react-dom'
 *
 * type Model = number
 * type Msg = 'INCREMENT' | 'DECREMENT'
 *
 * declare const init: [Model, cmd.none]
 * declare function update(msg: Msg, model: Model): [Model, cmd.Cmd<Msg>]
 * declare function view(model: Model): react.Html<Msg>
 *
 * const program = process.NODE_ENV === 'production' ? react.program : programWithDebugger
 *
 * const main = program(init, update, view)
 *
 * react.run(main, dom => render(document.getElementById('app')))
 * ```
 */
Object.defineProperty(exports, "__esModule", { value: true });
var IO_1 = require("fp-ts/lib/IO");
var Option_1 = require("fp-ts/lib/Option");
var pipeable_1 = require("fp-ts/lib/pipeable");
var rxjs_1 = require("rxjs");
var cmd = require("../Cmd");
var Html_1 = require("../Html");
var commons_1 = require("./commons");
var console_1 = require("./console");
var redux_devtool_1 = require("./redux-devtool");
/**
 * Adds a debugging capability to a generic `Html` `Program`.
 *
 * It tracks every `Message` dispatched and resulting `Model` update.
 *
 * It also lets directly updating the application's state with a special `Message` of type:
 *
 * ```ts
 * {
 *   type: '__DebugUpdateModel__'
 *   payload: Model
 * }
 * ```
 *
 * or applying a message with:
 * ```ts
 * {
 *   type: '__DebugApplyMsg__';
 *   payload: Msg
 * }
 * ```
 * @since 0.5.0
 */
function programWithDebugger(init, update, view, subscriptions) {
    var debug = runDebugger(window);
    var initModel = init[0];
    var data$ = new rxjs_1.BehaviorSubject([commons_1.debugInit(), initModel]);
    var updateWithDebug = function (msg, model) {
        if ('type' in msg) {
            switch (msg.type) {
                case '__DebugUpdateModel__':
                    return [msg.payload, cmd.none];
                case '__DebugApplyMsg__':
                    return [update(msg.payload, model)[0], cmd.none];
            }
        }
        var result = update(msg, model);
        data$.next([commons_1.debugMsg(msg), result[0]]);
        return result;
    };
    var p = Html_1.program(init, updateWithDebug, view, subscriptions);
    // --- Run the debugger
    debug({ data$: data$, init: initModel, dispatch: p.dispatch })();
    return p;
}
exports.programWithDebugger = programWithDebugger;
/**
 * Same as `programWithDebugger()` but with `Flags` that can be passed when the `Program` is created in order to manage initial values.
 * @since 0.5.0
 */
function programWithDebuggerWithFlags(init, update, view, subscriptions) {
    return function (flags) { return programWithDebugger(init(flags), update, view, subscriptions); };
}
exports.programWithDebuggerWithFlags = programWithDebuggerWithFlags;
/**
 * Checks which type of debugger can be used (standard `console` or _Redux DevTool Extension_) based on provided `window` and prepares the subscription to the "debug" stream
 * @since 0.5.0
 */
function runDebugger(win) {
    return function (deps) {
        return pipeable_1.pipe(redux_devtool_1.getConnection(win), IO_1.map(Option_1.fold(function () { return console_1.consoleDebugger(); }, redux_devtool_1.reduxDevToolDebugger)), IO_1.chain(function (debug) { return function () { return deps.data$.subscribe(debug(deps)); }; }));
    };
}
//# sourceMappingURL=index.js.map