"use strict";
/**
 * @file Integration with _Redux DevTool Extension_.
 *
 * Please check the [docs](https://github.com/zalmoxisus/redux-devtools-extension/tree/master/docs/API) fur further information.
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var Apply_1 = require("fp-ts/lib/Apply");
var E = require("fp-ts/lib/Either");
var IO_ = require("fp-ts/lib/IO");
var O = require("fp-ts/lib/Option");
var pipeable_1 = require("fp-ts/lib/pipeable");
var sequenceTEither = Apply_1.sequenceT(E.either);
/**
 * Gets a _Redux DevTool Extension_ connection in case the extension is available
 * @since 0.5.0
 */
function getConnection(global) {
    return function () { return (hasExtension(global) ? O.some(global.__REDUX_DEVTOOLS_EXTENSION__.connect()) : O.none); };
}
exports.getConnection = getConnection;
/**
 * **[UNSAFE]** Type guard to check if _Redux DevTool Extension_ is available.
 *
 * This is "tagged" as unsafe because the check is really loose.
 * @since 0.5.0
 */
function hasExtension(global) {
    return '__REDUX_DEVTOOLS_EXTENSION__' in global;
}
/**
 * **[UNSAFE]** Debug through _Redux DevTool Extension_
 * @since 0.5.0
 */
function reduxDevToolDebugger(connection) {
    return function (d) {
        var deps = __assign({}, d, { connection: connection });
        // --- Subscribe to extension in order to receive messages from monitor
        connection.subscribe(handleSubscription(deps));
        return handleActions(deps);
    };
}
exports.reduxDevToolDebugger = reduxDevToolDebugger;
/**
 * **[UNSAFE]** Handles the execution of an effect related to an incoming messages sent from extension monitor.
 *
 * @since 0.5.0
 */
function handleSubscription(deps) {
    var handler = handleIncomingMsg(deps);
    return function (msg) {
        return pipeable_1.pipe(handler(msg), E.fold(function (err) { return console.warn('[REDUX DEV TOOL]', err); }, function (eff) { return eff(); }));
    };
}
/**
 * **[UNSAFE]** Handles incoming messages sent from extension monitor.
 *
 * This is largely inspired by https://github.com/zalmoxisus/mobx-remotedev/blob/master/src/monitorActions.js
 *
 * **Note:** the monitor can dispatch messages of any shape that will be re-dispatched into the application; these messages **are not validated** and can lead to unexpected behaviours.
 * @since 0.5.0
 */
function handleIncomingMsg(_a) {
    var connection = _a.connection, init = _a.init, data$ = _a.data$, dispatch = _a.dispatch;
    var dispatchToApp = function (m) { return function () { return dispatch(m); }; };
    var reinit = function () { return connection.init(init); };
    var update = function (payload) { return dispatchToApp({ type: '__DebugUpdateModel__', payload: payload }); };
    var restart = function (model) { return function () { return connection.init(model); }; };
    var liftState = function (state) { return function () { return connection.send(null, state); }; };
    var toggle = toggleAction(dispatch);
    return function (msg) {
        switch (msg.type) {
            case 'START':
                return E.right(reinit);
            case 'ACTION':
                return pipeable_1.pipe(E.parseJSON(msg.payload, E.toError), E.bimap(function (e) { return e.message; }, dispatchToApp));
            case 'DISPATCH':
                switch (msg.payload.type) {
                    case 'JUMP_TO_STATE':
                    case 'JUMP_TO_ACTION':
                        return pipeable_1.pipe(parseJump(msg), E.map(update));
                    case 'RESET':
                        return E.right(pipeable_1.pipe(update(init), IO_.chain(function () { return reinit; })));
                    case 'ROLLBACK':
                        return pipeable_1.pipe(parseRollback(msg), E.map(function (m) {
                            return pipeable_1.pipe(update(m), IO_.chain(function () { return restart(m); }));
                        }));
                    case 'COMMIT':
                        return E.right(restart(data$.getValue()[1]));
                    case 'IMPORT_STATE':
                        return pipeable_1.pipe(parseImportState(msg), E.map(function (liftedState) {
                            return pipeable_1.pipe(update(liftedState.computedStates[liftedState.computedStates.length - 1].state), IO_.chain(function () { return liftState(liftedState); }));
                        }));
                    case 'TOGGLE_ACTION':
                        return pipeable_1.pipe(parseToggleAction(msg), E.map(function (_a) {
                            var id = _a[0], liftedState = _a[1];
                            return pipeable_1.pipe(toggle(id, liftedState), IO_.chain(liftState));
                        }));
                    default:
                        return E.left(msg.payload.type);
                }
        }
    };
}
/**
 * Handles debugging actions.
 *
 * The `MESSAGE` action will send the message payload and state to the connected extension (`connection`).
 * @since 0.5.0
 */
function handleActions(_a) {
    var connection = _a.connection;
    return function (_a) {
        var action = _a[0], model = _a[1];
        return (action.type === 'MESSAGE' ? connection.send(action.payload, model) : undefined);
    };
}
/**
 * Parses a `JUMP` message.
 *
 * @since 0.5.0
 */
function parseJump(msg) {
    return pipeable_1.pipe(E.parseJSON(msg.state, E.toError), E.bimap(function (e) { return e.message; }, function (u) { return u; }));
}
/**
 * Parses a `ROLLBACK` message.
 *
 * @since 0.5.0
 */
function parseRollback(msg) {
    return pipeable_1.pipe(E.parseJSON(msg.state, E.toError), E.bimap(function (e) { return e.message; }, function (u) { return u; }));
}
/**
 * Parses an `IMPORT_STATE` message.
 *
 * @since 0.5.0
 */
function parseImportState(msg) {
    return typeof msg.payload.nextLiftedState === 'object' && msg.payload.nextLiftedState !== null
        ? E.right(msg.payload.nextLiftedState)
        : E.left('IMPORT_STATE message has some bad payload...');
}
/**
 * Parses a `TOGGLE_ACTION` message.
 *
 * @since 0.5.0
 */
function parseToggleAction(msg) {
    var getId = pipeable_1.pipe(msg.payload.id, E.fromNullable('TOGGLE_ACTION message has some bad payload...'), E.map(function (x) { return x; }));
    var parseState = pipeable_1.pipe(E.parseJSON(msg.state, E.toError), E.bimap(function (e) { return e.message; }, function (u) { return u; }));
    return sequenceTEither(getId, parseState);
}
/**
 * Handles toggling a specific action (identified by `id` parameter).
 *
 * It re-executes all the actions (and updates the store) excluding the toggled one.
 *
 * The implementation is taken from [MobX dev tool integration](https://github.com/zalmoxisus/mobx-remotedev/blob/master/src/monitorActions.js#L22)
 *
 * @since 0.5.0
 */
function toggleAction(dispatch) {
    return function (id, liftedState) { return function () {
        var state = JSON.parse(JSON.stringify(liftedState)); // poor man deep clone...
        var skippedActionIds = state.skippedActionIds, stagedActionIds = state.stagedActionIds, computedStates = state.computedStates, actionsById = state.actionsById;
        var skippedIndex = skippedActionIds.indexOf(id);
        var skipped = skippedIndex !== -1;
        var actionIndex = stagedActionIds.indexOf(id);
        if (actionIndex === -1) {
            return state;
        }
        dispatch({ type: '__DebugUpdateModel__', payload: computedStates[actionIndex - 1].state });
        var start = skipped ? actionIndex : actionIndex + 1;
        var end = stagedActionIds.length;
        for (var i = start; i < end; i++) {
            var currentActionId = stagedActionIds[i];
            if (i !== actionIndex && skippedActionIds.indexOf(currentActionId) !== -1) {
                continue; // it's already skipped
            }
            dispatch({ type: '__DebugApplyMsg__', payload: actionsById[currentActionId].action });
        }
        if (skipped) {
            skippedActionIds.splice(skippedIndex, 1);
        }
        else {
            skippedActionIds.push(id);
        }
        return state;
    }; };
}
//# sourceMappingURL=redux-devtool.js.map