"use strict";
/**
 * @file A specialization of `Program` that handles application navigation via location's hash.
 *
 * It uses [`history`](https://github.com/ReactTraining/history) package
 */
Object.defineProperty(exports, "__esModule", { value: true });
var O = require("fp-ts/lib/Option");
var H = require("history");
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var html = require("./Html");
var Sub_1 = require("./Sub");
var history = H.createHashHistory();
/**
 * Location changes are expressed as a stream
 */
var location$ = new rxjs_1.Subject();
function getLocation() {
    return history.location;
}
history.listen(function (location) {
    location$.next(location);
});
/**
 * Generates a `Cmd` that adds a new location to the history's list.
 * @since 0.5.0
 */
function push(url) {
    return rxjs_1.of(function () {
        history.push(url);
        return Promise.resolve(O.none);
    });
}
exports.push = push;
/**
 * Returns a `Program` specialized for `Navigation`.
 *
 * The `Program` is a `Html.Program` but it needs a `locationToMsg()` function which converts location changes to messages.
 *
 * Underneath it consumes `location$` stream (applying `locationToMsg()` on its values).
 * @since 0.5.0
 */
function program(locationToMessage, init, update, view, subscriptions) {
    if (subscriptions === void 0) { subscriptions = function () { return Sub_1.none; }; }
    var onChangeLocation$ = location$.pipe(operators_1.map(function (location) { return locationToMessage(location); }));
    var subs = function (model) { return Sub_1.batch([subscriptions(model), onChangeLocation$]); };
    return html.program(init(getLocation()), update, view, subs);
}
exports.program = program;
/**
 * Same as `program()` but with `Flags` that can be passed when the `Program` is created in order to manage initial values.
 * @since 0.5.0
 */
function programWithFlags(locationToMessage, init, update, view, subscriptions) {
    if (subscriptions === void 0) { subscriptions = function () { return Sub_1.none; }; }
    return function (flags) { return program(locationToMessage, init(flags), update, view, subscriptions); };
}
exports.programWithFlags = programWithFlags;
//# sourceMappingURL=Navigation.js.map